import * as React from 'react';
import { Switch } from 'react-router-dom';
import { CSSTransition, TransitionGroup } from 'react-transition-group';
export const initSlideRouter = (userSetting) => {
    const { history } = userSetting;
    if (!history) {
        throw new Error('slider router history is missing');
    }
    settings = { ...settings, ...userSetting };
    const routeAnimationDuration = settings.routeAnimationDuration;
    const classNames = settings.classNames;
    const wrapId = settings.wrapId;
    const isRememberPosition = true;
    const baseStyle = document.createElement('style');
    baseStyle.innerHTML = `
         .${classNames}-enter{
    position: fixed;
    opacity: 0;
    transition : left 1s;
  }

     .${classNames}-enter-active{
    position: relative;
    opacity: 0;
    transition : left 1s;
  }
  .${classNames}-exit{
    position: relative;
    top: 0;
    left: 0;
    z-index: 1000;
        transition : left 1s;
  }
  .${classNames}-exit-active{
    position: relative;
    top: 0;
    left: 0;
    z-index: 1000;
        transition : left 1s;
  }
    `;
    document.body.appendChild(baseStyle);
    let lastPathname = history.location.pathname;
    const positionRecord = {};
    let isAnimating = false;
    let bodyOverflowX = '';
    let historyKeys = sessionStorage.getItem('historyKeys') ? JSON.parse(sessionStorage.getItem('historyKeys')) :
        history.location.key ? [history.location.key] : [''];
    let currentHistoryPosition = historyKeys.indexOf(history.location.key);
    currentHistoryPosition = currentHistoryPosition === -1 ? 0 : currentHistoryPosition;
    history.listen((() => {
        if (lastPathname === history.location.pathname) {
            return;
        }
        if (!history.location.key) {
            historyKeys[0] = '';
        }
        const delay = 50;
        if (!isAnimating) {
            bodyOverflowX = document.body.style.overflowX;
        }
        const routerWrap = document.getElementById(wrapId);
        const originPage = routerWrap.children[0];
        const oPosition = originPage.style.position;
        setTimeout(() => {
            document.body.style.overflowX = bodyOverflowX;
            originPage.style.position = oPosition;
            isAnimating = false;
        }, routeAnimationDuration + delay + 50);
        document.body.style.overflowX = 'hidden';
        if (history.location.state && history.location.state.noAnimate) {
            setTimeout(() => {
                const wrap = document.getElementById(wrapId);
                const newPage = wrap.children[0];
                const oldPage = wrap.children[1];
                newPage.style.opacity = '1';
                oldPage.style.display = 'none';
            });
            return;
        }
        const { action } = history;
        const currentRouterKey = history.location.key ? history.location.key : '';
        const isForward = historyKeys[currentHistoryPosition + 1] === currentRouterKey;
        const oldScrollPosition = window.scrollY;
        originPage.style.top = -oldScrollPosition + 'px';
        originPage.style.position = 'fixed';
        setTimeout(() => {
            isAnimating = true;
            const wrap = document.getElementById(wrapId);
            const newPage = wrap.children[0];
            const oldPage = wrap.children[1];
            if (!newPage || !oldPage) {
                return;
            }
            const currentPath = history.location.pathname;
            if (action === 'PUSH' || isForward) {
                window.scrollTo(0, 0);
                positionRecord[lastPathname] = oldScrollPosition;
                if (action === 'PUSH') {
                    historyKeys = historyKeys.slice(0, currentHistoryPosition + 1);
                    historyKeys.push(currentRouterKey);
                }
            }
            else {
                if (isRememberPosition) {
                    setTimeout(() => {
                        window.scrollTo(0, positionRecord[currentPath]);
                    }, 50);
                }
                for (const key in positionRecord) {
                    if (key === currentPath) {
                        continue;
                    }
                    if (key.startsWith(currentPath)) {
                        delete positionRecord[key];
                    }
                }
            }
            if (action === 'REPLACE') {
                historyKeys[currentHistoryPosition] = currentRouterKey;
            }
            sessionStorage.setItem('historyKeys', JSON.stringify(historyKeys));
            newPage.style.width = '100%';
            oldPage.style.width = '100%';
            newPage.style.top = '0px';
            if (action === 'PUSH' || isForward) {
                newPage.style.left = '100%';
                oldPage.style.left = '0';
                newPage.style.transition = `left ${(routeAnimationDuration - delay) / 1000}s`;
                newPage.style.webkitTransition = `left ${(routeAnimationDuration - delay) / 1000}s`;
                oldPage.style.transition = `left ${(routeAnimationDuration - delay) / 1000}s`;
                oldPage.style.webkitTransition = `left ${(routeAnimationDuration - delay) / 1000}s`;
                setTimeout(() => {
                    newPage.style.opacity = '1';
                    newPage.style.left = '0';
                    oldPage.style.left = '-100%';
                }, delay);
            }
            else {
                newPage.style.left = '-100%';
                oldPage.style.left = '0';
                setTimeout(() => {
                    oldPage.style.transition = `left ${(routeAnimationDuration - delay) / 1000}s`;
                    oldPage.style.webkitTransition = `left ${(routeAnimationDuration - delay) / 1000}s`;
                    newPage.style.transition = `left ${(routeAnimationDuration - delay) / 1000}s`;
                    newPage.style.webkitTransition = `left ${(routeAnimationDuration - delay) / 1000}s`;
                    newPage.style.left = '0';
                    oldPage.style.left = '100%';
                    newPage.style.opacity = '1';
                }, delay);
            }
            currentHistoryPosition = historyKeys.indexOf(currentRouterKey);
            lastPathname = history.location.pathname;
        }, 50);
    }));
};
let settings = {
    wrapId: 'slide-router-wrap',
    classNames: 'slide-router',
    routeAnimationDuration: 350,
    history: null
};
export default class SlideRouter extends React.Component {
    render() {
        const { location } = settings.history;
        const { routeAnimationDuration, wrapId, classNames, transitionProps } = settings;
        return (React.createElement(React.Fragment, null,
            React.createElement(TransitionGroup, { id: wrapId },
                React.createElement(CSSTransition, Object.assign({ classNames: classNames, timeout: routeAnimationDuration, key: location.pathname }, transitionProps),
                    React.createElement(Switch, { location: location }, this.props.children)))));
    }
}
//# sourceMappingURL=index.js.map